@@ -46,9 +46,13 @@ AmConf['ConfHome'] = AmConf['MailHome'] + '/conf';
 # =====================================================================
 #
 AmConf['OccurenceStore'] = AmConf['ConfHome'] + '/feml_store.pstore';
+
 AmConf['UfromWhiteList'] = AmConf['ConfHome'] + '/ufrom_white_list.txt';
 AmConf['FromWhiteList'] = AmConf['ConfHome'] + '/from_white_list.txt';
 AmConf['ReceivedWhiteList'] = AmConf['ConfHome'] + '/received_white_list.txt';
+AmConf['UfromBlackList'] = AmConf['ConfHome'] + '/ufrom_black_list.txt';
+AmConf['FromBlackList'] = AmConf['ConfHome'] + '/from_black_list.txt';
+AmConf['ReceivedBlackList'] = AmConf['ConfHome'] + '/received_black_list.txt';
 
 # =====================================================================
 #
@@ -57,12 +61,14 @@ AmConf['BackupMailDir'] = AmConf['MailHome'] + '/backup';
 AmConf['CleanMailDir'] = AmConf['MailHome'] + '/clean';
 AmConf['WhiteMailDir'] = AmConf['MailHome'] + '/white';
 AmConf['DirtyMailDir'] = AmConf['MailHome'] + '/dirty';
+AmConf['BlackMailDir'] = AmConf['MailHome'] + '/black';
 
 # =====================================================================
 #
 ##AmConf['AddHeaderIfClean'] = "X-Feml: clean";
 ##AmConf['AddHeaderIfWhite'] = "X-Feml: white";
 ##AmConf['AddHeaderIfDirty'] = "X-Feml: dirty";
+##AmConf['AddHeaderIfBlack'] = "X-Feml: black";
 		# ƤСȽ̤˱ƥإåɲäƽϤ롣
 
 # =====================================================================
@@ -85,6 +91,13 @@ AmConf['SpamIfIllegalDate'] = 'on';
 		#  (Date:) ʤ硢ѥȴ
 		# -- ʸȤƲԲ
 		# -- Ȥκ1.5ʾ
+AmConf['SpamIfShiftJisSubject'] = 'on';
+		# Subject:׹ԤShift-JISʸޤƤ硢
+		# ѥȴ
+		#	EUC-JPʤɤ⤽ΤޤΤϵʰȿ
+		#	ºݤܤˤĤΤShift-JISФꡣ
+		# ## ISO-8859-1ʤɤʸShift-JISȸǧǽ
+		# ## 뤫Τʤ
 
 AmConf['SpamCharsets'] = 'utf-7,unicode-1-1-utf-7,gb2312,gb18030,koi8,koi8-r,koi8-u';
 		# 沽ѤƤΤϥѥȴ
@@ -104,7 +117,7 @@ AmConf['SpamThreshold'] = 0.95;
 # ---------------------------------------------------------------------
 # ---------------------------------------------------------------------
 #
-FemlVersion = 'feml-0.3.0';
+FemlVersion = 'feml-0.4.0';
 
 # =====================================================================
 # ֡뷿ܤˤĤơˤʤäƤ뤫ɤĴ٤롣
@@ -114,6 +127,15 @@ def is_conf_on(key)
 	return (AmConf[key] != nil && AmConf[key] != '');
 end
 
+# =====================================================================
+# ΡʸɤȽꤹ롣
+#	MIMEǥɤΡʸˤĤȽꤷΤǡ
+#	=?פƤ Kconv::guess() Ƚꤹ롣
+#
+def Kconv::raw_guess(str)
+	return Kconv::guess(str.gsub(/=\?/, ''));
+end
+
 # =====================================================================
 # ѥΨȽ
 #
@@ -362,6 +384,8 @@ class ExDecoder
 	# Żҥ᡼ΥƥȤե뤫ɤ߹ࡣ
 	#
 	def load_fp(fp)
+		fp.set_encoding('ASCII-8BIT');
+					# ʳǤϥ󥳡ǥ󥰤
 		@mbuf_array = fp.readlines();
 		parse_header();
 		parse_body();
@@ -371,7 +395,12 @@ class ExDecoder
 	# Żҥ᡼ΥƥȤʸȤɤ߹ࡣ
 	#
 	def load_array(buf_array)
-		@mbuf_array = buf_array;
+		@mbuf_array = Array.new();
+		buf_array.each do |buf|
+			buf.force_encoding('ASCII-8BIT');
+					# ʳǤϥ󥳡ǥ󥰤
+			@mbuf_array << buf;
+		end
 		parse_header();
 		parse_body();
 	end
@@ -519,7 +548,7 @@ class ExDecoder
 			0xDD => 'Y',		# &Yacute;
 			0xDE => 'P',		# &THORN;
 						# PפѤȤƻȤäƤ
-						# 㤬¿褦˸롣
+						# 㤬¿
 			0xDF => 'ss',		# &szlig;
 			0xE0 => 'a',		# &agrave;
 			0xE1 => 'a',		# &aacute;
@@ -710,10 +739,6 @@ class ExDecoder
 			# -e: Output code is EUC
 			# -I: Convert non ISO-2022-JP character to GETA
 			# -m0: No MIME decode
-			#	nkfǤMIME󥳡ɤ줿ʬľˤ
-			#	Ԥ褦ʤΤǡ򤱤롣
-			#	̾ʻͤǤ뤬ɤΤ褦
-			#	󥳡ɤƤ뤫ѻŪˤŬڡ
 			#	nkf̤бcharsetΤᡢˤƤ
 			#	MIMEǥɤνɬפˤʤ롣
 			# --numchar-input: Convert Unicode Character Reference
@@ -721,14 +746,20 @@ class ExDecoder
 
 		# -----------------------------------------------------
 		# [MIME-B] (base64)[MIME-Q] (quoted-printable) Υǥɡ
+		#	=?<charset>?[BQ]?<encoded_string>?=
+		#		θˤMIME󥳡ʸ󤬤С
+		#		ľζ롣Τᡢ
+		#			( +(?==\?))?
+		#		ˤꡢɤʬˡ=?פΤߡ
+		#		¤ӤޤǾȹ礹롣
 		#
-		mbuf = mbuf.gsub(/=\?(.+?)\?B\?([!->@-~]+?)\?=/i) do
+		mbuf = mbuf.gsub(/=\?(.+?)\?B\?([!->@-~]+?)\?=( +(?==\?))?/i) do
 			m_charset = $1 || '';
 			text = $2;
 			s = buf_decode(text.unpack("m")[0], m_charset);
 			s;
 		end
-		mbuf = mbuf.gsub(/=\?(.+?)\?Q\?(.+?)\?=/im) do
+		mbuf = mbuf.gsub(/=\?(.+?)\?Q\?(.+?)\?=( +(?==\?))?/im) do
 			m_charset = $1 || '';
 			text = $2;
 			s = buf_decode(text.unpack("M")[0], m_charset);
@@ -748,30 +779,38 @@ class ExDecoder
 		# - ǥɤEUC-JPɸಽ롣
 		#
 		header_body_delim_i = nil;
-		@mbuf_array.each_index  do |i|
+		@mbuf_array.each_index do |i|
 			mbuf = @mbuf_array[i];
 			if mbuf == nil || mbuf == "\n"		## 
 				header_body_delim_i = i;
 				break;
 			end
-			if /^\s/ !~ mbuf.toeuc
+			if /^\s/ !~ mbuf
 				@mbuf_header_array << mbuf;
 			else				## ³
 				@mbuf_header_array[@mbuf_header_array.size - 1] += mbuf;
 			end
 		end
+
+		@from_raw_enc = Kconv::UNKNOWN;
+		@subject_raw_enc = Kconv::UNKNOWN;
 		@mbuf_header_array.each_index do |i|
 			mbuf = @mbuf_header_array[i];
 			m_charset = 'iso-8859-1';
-			if /^(From|Subject):/ =~ mbuf.toeuc
+			if /^From:/i =~ mbuf
 				m_charset = nil;
+				@from_raw_enc = Kconv::raw_guess(mbuf);
+			end
+			if /^Subject:/i =~ mbuf
+				m_charset = nil;
+				@subject_raw_enc = Kconv::raw_guess(mbuf);
 			end
 			@mbuf_header_array[i] = buf_decode(mbuf, m_charset);
 		end
-				# MIME󥳡ɤʤɤ³Ԥˤޤ³Ƥ
-				# ȤΤǡϢ뤷ǥǥɤ롣
+				# MIME-Q󥳡ɤ³Ԥˤޤ³Ƥ
+				# 礬ΤǡϢ뤷ǥǥɤ롣
 				# إåΥɤiso-8859-1§
-				# Shift-JISΤޤޤƤΤ
+				# Shift-JISΤޤޤƤ㤬
 				# ¿ΤǡSubject/FromԤˤĤƤ
 				# 㳰Ȥư
 				#
@@ -820,7 +859,7 @@ class ExDecoder
 			b_regex = Regexp.new("--#{Regexp.escape(@boundary)}(--)?$");
 			end_boundary_appeared = false;
 			@mbuf_body_array.each_index do |i|
-				mbuf = @mbuf_body_array[i].toeuc;
+				mbuf = @mbuf_body_array[i];
 				if b_regex =~ mbuf
 					end_boundary_appeared = true if $1 == '--';
 					@boundary_lines << i;
@@ -833,6 +872,8 @@ class ExDecoder
 			end
 		end
 
+##		print "[from_raw_enc] #{@from_raw_enc.inspect}\n";
+##		print "[subject_raw_enc] #{@subject_raw_enc.inspect}\n";
 ##		print "[encoding] #{@encoding}\n";
 ##		print "[content_type_major] #{@content_type_major}\n";
 ##		print "[content_type_minor] #{@content_type_minor}\n";
@@ -843,6 +884,28 @@ class ExDecoder
 	end
 	private :parse_header;
 
+	# =============================================================
+	# Subject:׹ԤΥ󥳡ǥ (MIMEǥɤ) ֤
+	#	Shift-JISʸ󤬤ΤޤޤƤ㤬¿Τǡ
+	#	ȽȤѤ롣
+	#
+	def raw_header_encoding()
+		kconv_name_tbl = {
+			Kconv::ASCII => 'ascii',
+			Kconv::EUC => 'euc-jp',
+			Kconv::JIS => 'iso-2022-jp',
+			Kconv::SJIS => 'shift_jis',
+			Kconv::UTF8 => 'utf-8',
+			Kconv::UTF16 => 'utf-16',
+			Kconv::UTF32 => 'utf-32',
+			Kconv::BINARY => 'binary',
+			Kconv::UNKNOWN => 'unknown',
+		};
+		s = kconv_name_tbl[@subject_raw_enc] || 'unknown';
+
+		return s;
+	end
+
 	# =============================================================
 	# إåƤ; Ȥmultipartξꡣ
 	#
@@ -928,8 +991,8 @@ class ExDecoder
 		@mbuf_body = '';
 		tbuf = '';
 
-		top_line = (@mbuf_body_array[0] || '').toeuc;
-		if /^MAIL FROM:/ =~ top_line
+		top_line = (@mbuf_body_array[0] || '');
+		if /^(MAIL FROM:|QUIT)/ =~ top_line
 			sub_ex_d = ExDecoder.new();
 			sub_ex_d.load_array(@mbuf_body_array);
 			@mbuf_body += 'X-From: ';
@@ -940,7 +1003,7 @@ class ExDecoder
 			@mbuf_body += sub_ex_d.get_decoded_body;
 				#
 				# (ʷ)
-				# ʸˡMAIL FROM:פϤޤ
+				# ʸˡMAIL FROM:פQUITפϤޤ
 				# SMTPȤΤȤ꤬äƤ롣
 				# ʷǤϤ뤬Τ褦ʥѥब
 				# ¿Τǡ̰ƲϤ롣
@@ -992,7 +1055,7 @@ class ExDecoder
 	protected :parse_body;
 
 	# =============================================================
-	# إåͤ롣
+	# إå(ǥɺѤ)롣
 	# :
 	#	s = dec.get_header('Subject:')	# => (Subject:׹Ԥ)
 	# إåʤnil
@@ -1007,6 +1070,39 @@ class ExDecoder
 		return nil;
 	end
 
+	# =============================================================
+	#
+	def get_unix_from()
+		unix_from_line = get_header('From ') || '';
+		unix_from = '';
+		if /^(\S+) / =~ unix_from_line
+			unix_from = $1;
+		end
+		return unix_from;
+	end
+
+	# =============================================================
+	#
+	def get_mail_from()
+		from_header = get_header('From:') || '';
+
+		if /<(.+)>/ =~ from_header
+			mail_addr = $1;
+		elsif /([^ ]+)\s*\(.*\)/ =~ from_header
+			mail_addr = $1;
+		else
+			mail_addr = from_header.strip;
+		end
+
+		local_part_chars = '[0-9A-Za-z_!#\$%&\'*+\-\/=\?^_{|}\~\.]+';
+		domain_part_chars = '[0-9A-Za-z_\-\.]+';
+		valid_pattern = Regexp::new("^(#{local_part_chars})@(#{domain_part_chars})$".force_encoding('ASCII-8BIT'));
+		if valid_pattern !~ mail_addr
+			mail_addr = '';
+		end
+		return mail_addr;
+	end
+
 	# =============================================================
 	# Żҥ᡼ʸ(ǥɺѤ)֤
 	#
@@ -1232,33 +1328,24 @@ class Tokenizer
 
 end
 
-# =====================================================================
-# ᡼롦ɥ쥹ʸȤƤǧ롣
-#
-def mail_addr_verify(mail_addr)
-	local_part_chars = '[0-9A-Za-z_!#\$%&\'*+\-\/=\?^_{|}\~\.]+';
-	domain_part_chars = '[0-9A-Za-z_\-\.]+';
-		# \wפȽ񤯤ȤʸޤޤΤǡ
-		# 0-9A-Za-z_ ȳƵҡ
-	valid_pattern = Regexp::new("^(#{local_part_chars})@(#{domain_part_chars})$".force_encoding('ASCII-8BIT'));
-	return (valid_pattern =~ mail_addr) ? true : false;
-end
-
 # ================================================================
-# From:׹Ԥ᡼롦ɥ쥹Ф롣
+# Хåå¸¾ɬפʥǥ쥯ȥ꡼Ѱդ롣
 #
-def extract_mail_addr(buf)
-	if /<(.+)>/ =~ buf
-		mail_addr = $1;
-	elsif /([^ ]+)\s*\(.*\)/ =~ buf
-		mail_addr = $1;
-	else
-		mail_addr = buf.strip;
+def make_directories()
+	[ 'LogFile', 'OccurenceStore' ].each do |f|
+		file = AmConf[f];
+		dir = File::dirname(file);
+		if ! File::directory?(dir)
+			FileUtils::mkdir_p(dir);
+		end
 	end
-	if mail_addr_verify(mail_addr) == false
-		mail_addr = '';
+
+	[ 'BackupMailDir', 'CleanMailDir', 'WhiteMailDir', 'DirtyMailDir', 'BlackMailDir' ].each do |d|
+		dir = AmConf[d];
+		if ! File::directory?(dir)
+			FileUtils::mkdir_p(dir);
+		end
 	end
-	return mail_addr;
 end
 
 # ================================================================
@@ -1289,28 +1376,44 @@ def verify_date(date_line)
 end
 
 # ================================================================
-# ۥ磻ȡꥹ (ե˵) ˹פƤ뤫ݤθ
+# ۥ磻ȡꥹ/֥åꥹȤ˹פƤ뤫ݤθ
 # target_text: оʸ (̤ˤإåʸ)
-# white_list_class: ե AmConf[white_list_class] ˵Ҥ줿
-#		ۥ磻ȡꥹȤtarget_textȤ
+# list_class: ե AmConf[list_class] ˵Ҥ줿ꥹȤ
+#		target_textȤ
 #
-def is_in_white_list_pattern(target_text, white_list_class)
-	white_list_file = AmConf[white_list_class];
+def is_in_list_pattern(target_text, list_class)
+	list_file = AmConf[list_class];
 
-	return false if white_list_file == nil;
-	return false if ! File.file?(white_list_file);
+	return false if list_file == nil;
+	return false if ! File.file?(list_file);
 
-	File::foreach(white_list_file) do |buf|
+	File::foreach(list_file) do |buf|
+		buf.force_encoding('ASCII-8BIT');
 		buf = buf.chomp();
 		next if /^#/ =~ buf;
 		next if /^\*/ =~ buf;
 		next if /^\s*$/ =~ buf;
-		buf.force_encoding('ASCII-8BIT');
-		begin
-			pattern = Regexp::new(buf);
-			return true if pattern =~ target_text;
-		rescue RegexpError
-			## ɽȤ
+
+		pattern_string = nil;
+		if %r|^/(.+)/$| =~ buf
+			pattern_string = $1;
+		end
+						# %r ˡޤǤȤ߹ޤʤ
+
+		pattern = nil;
+		if pattern_string != nil
+			begin
+				pattern = Regexp::new(pattern_string);
+			rescue RegexpError
+				## ɽȤ
+			end
+		else
+			pattern = buf;
+		end
+		next if pattern == nil;
+
+		if target_text.index(pattern) != nil
+			return true;
 		end
 	end
 	return false;
@@ -1346,17 +1449,8 @@ def spam_filter(fp_in, user)
 
 	# ================================================================
 	#
-	dir = File::dirname(AmConf['OccurenceStore']);
-	if ! File::directory?(dir)
-		FileUtils::mkdir_p(dir);
-	end
+	make_directories();
 
-	[ 'BackupMailDir', 'CleanMailDir', 'WhiteMailDir', 'DirtyMailDir' ].each do |d|
-		dir = AmConf[d];
-		if ! File::directory?(dir)
-			FileUtils::mkdir_p(dir);
-		end
-	end
 	timestamp = Time::now.strftime("%Y%m%d_%H%M%S") + sprintf("_%05d", $$);
 	mail_file_name = "mail_#{timestamp}.txt";
 
@@ -1366,6 +1460,7 @@ def spam_filter(fp_in, user)
 	# - From סFrom:׹Ԥ᡼롦ɥ쥹ФƤ
 	# - Τ˶ǤʤԤ뤫ɤĴ٤롣
 	# - դɤĴ٤롣
+	# - Subject:׹ԤΥ󥳡ǥ󥰤Ĵ٤롣
 	# - ȤƤcharset󤷤Ƥ (multipartǤʣˤʤꤦ)
 	#
 	exd = ExDecoder.new();
@@ -1378,19 +1473,15 @@ def spam_filter(fp_in, user)
 	lfp.print "\n";
 	lfp.print "** #{timestamp} ----------------------------------------\n";
 
-	unix_from_line = exd.get_header('From ') || '';
-	from_header = exd.get_header('From:') || '';
+	unix_from = exd.get_unix_from();
+	mail_from = exd.get_mail_from();
+
 	subject = exd.get_header('Subject:') || '';
 	date_header = exd.get_header('Date:') || '';
 	received = exd.get_header('Received:') || '';
 			# 'Received:' إåʣ硢1Ĥ᤬롣
 	mail_body = exd.get_decoded_body() || '';
-
-	unix_from = '';
-	if /^(\S+) / =~ unix_from_line
-		unix_from = $1;
-	end
-	mail_address = extract_mail_addr(from_header);
+	header_enc = exd.raw_header_encoding;
 
 	date_diag = verify_date(date_header);
 
@@ -1411,54 +1502,85 @@ def spam_filter(fp_in, user)
 		end
 	end
 
+	s_ch = '';
+	if (spam_charsets = AmConf['SpamCharsets']) != nil
+		spam_charsets_array = spam_charsets.split(/,\s*/);
+		s_ch = (spam_charsets_array & charset_used).join(',');
+						# ѥȴcharset
+	end
+
 	# ================================================================
 	# SPAMȽꡣ
 	#
 	sp = SpamProb.new(subject + "\n" + mail_body, lfp);
 	spam_prob = sp.probability;
-
-	spam_charsets_array = nil;
-	if (spam_charsets = AmConf['SpamCharsets']) != nil
-		spam_charsets_array = spam_charsets.split(/,\s*/);
-	end
+	spam_prob_percent = sprintf("%.2f%%", spam_prob * 100);
 
 	result = '';
-	is_in_white_list = false;
-	spam_prob_percent = sprintf("%.2f%%", spam_prob * 100);
-	if is_in_white_list_pattern(unix_from, 'UfromWhiteList')
+	result_mode = :unknown;
+
+	if is_in_list_pattern(unix_from, 'UfromWhiteList')
 		result = "FemlWhite (unix_from) (#{spam_prob_percent})";
-		is_in_white_list = true;
-	elsif is_in_white_list_pattern(mail_address, 'FromWhiteList')
+		result_mode = :is_white;
+	elsif is_in_list_pattern(mail_from, 'FromWhiteList')
 		result = "FemlWhite (mail_from) (#{spam_prob_percent})";
-		is_in_white_list = true;
-	elsif is_in_white_list_pattern(received, 'ReceivedWhiteList')
+		result_mode = :is_white;
+	elsif is_in_list_pattern(received, 'ReceivedWhiteList')
 		result = "FemlWhite (received) (#{spam_prob_percent})";
-		is_in_white_list = true;
+		result_mode = :is_white;
+
+	elsif is_in_list_pattern(unix_from, 'UfromBlackList')
+		result = "FemlBlack (unix_from) [#{spam_prob_percent}]";
+		result_mode = :is_black;
+	elsif is_in_list_pattern(mail_from, 'FromBlackList')
+		result = "FemlBlack (mail_from) [#{spam_prob_percent}]";
+		result_mode = :is_black;
+	elsif is_in_list_pattern(received, 'ReceivedBlackList')
+		result = "FemlBlack (received) [#{spam_prob_percent}]";
+		result_mode = :is_black;
+
 	elsif is_conf_on('SpamIfEmptyBody') && ! non_empty_line_found
 		result = "FemlDirty (empty body) [#{spam_prob_percent}]";
-		spam_prob = 1.0;
+		result_mode = :is_dirty;
 	elsif is_conf_on('SpamIfIllegalDate') && date_diag != ''
 		result = "FemlDirty (date line) [#{spam_prob_percent}] #{date_diag}";
-		spam_prob = 1.0;
-	elsif spam_charsets_array != nil &&
-	      ! (s_ch = (spam_charsets_array & charset_used).join(',')).empty?
+		result_mode = :is_dirty;
+	elsif is_conf_on('SpamIfShiftJisSubject') && header_enc == 'shift_jis'
+		result = "FemlDirty (SJIS subject) [#{spam_prob_percent}]";
+		result_mode = :is_dirty;
+
+	elsif ! s_ch.empty?
 		result = "FemlDirty (charset: #{s_ch}) [#{spam_prob_percent}]";
-		spam_prob = 1.0;
+		result_mode = :is_dirty;
+
 	elsif spam_prob < AmConf['SpamThreshold'].to_f
 		result = "FemlClean (#{spam_prob_percent})";
+		result_mode = :is_clean;
 	else
 		result = "FemlDirty [#{spam_prob_percent}]";
+		result_mode = :is_dirty;
 	end
 
 	# ==============================================================
 	# ۥ磻ȡꥹȤˤˤ⤫餺SPAMȽꤵƤޤäϡ
 	# ǡ١롣
 	#
-	if is_in_white_list && AmConf['SpamThreshold'].to_f <= spam_prob
+	if result_mode == :is_white &&
+	   AmConf['SpamThreshold'].to_f <= spam_prob
 		tk = Tokenizer.new(subject + "\n" + mail_body);
 		tk.update_occurences(:add_clean_sub_spam);
 	end
 
+	# ==============================================================
+	# ֥åꥹȤˤ/¾δˤꥹѥȽꤷˤ⤫餺
+	# SPAMȽꤵʤä⡢ǡ١롣
+	#
+	if (result_mode == :is_dirty || result_mode == :is_black) &&
+	   spam_prob < AmConf['SpamThreshold'].to_f
+		tk = Tokenizer.new(subject + "\n" + mail_body);
+		tk.update_occurences(:add_spam_sub_clean);
+	end
+
 	# ================================================================
 	# ˾Ͽ롣
 	#
@@ -1466,7 +1588,7 @@ def spam_filter(fp_in, user)
 	lfp.print <<-"__LOG__";
 ** #{timestamp} ----------------------------------------
 ** #{result}
-** FromLine: #{mail_address} / UnixFrom: #{unix_from} / charset: #{charset_used.join(',')}
+** FromLine: #{mail_from} / UnixFrom: #{unix_from} / charset: #{charset_used.join(',')} / header_enc: #{header_enc}
 ** SubjectLine: #{subject.chomp}
 
 	__LOG__
@@ -1475,16 +1597,20 @@ def spam_filter(fp_in, user)
 
 	# ================================================================
 	#
-	if is_in_white_list
-		dir_mode = 'WhiteMailDir';
-		add_header_mode = 'AddHeaderIfWhite';
-	elsif spam_prob < AmConf['SpamThreshold'].to_f
-		dir_mode = 'CleanMailDir';
-		add_header_mode = 'AddHeaderIfClean';
-	else
-		dir_mode = 'DirtyMailDir';
-		add_header_mode = 'AddHeaderIfDirty';
-	end
+	dir_mode_s = {
+		:is_white => 'WhiteMailDir',
+		:is_dirty => 'DirtyMailDir',
+		:is_clean => 'CleanMailDir',
+		:is_black => 'BlackMailDir',
+	};
+	add_header_mode_s = {
+		:is_white => 'AddHeaderIfWhite',
+		:is_dirty => 'AddHeaderIfDirty',
+		:is_clean => 'AddHeaderIfClean',
+		:is_black => 'AddHeaderIfBlack',
+	};
+	dir_mode = dir_mode_s[result_mode];
+	add_header_mode = add_header_mode_s[result_mode];
 
 	exd.store(AmConf[dir_mode] + '/' + mail_file_name);
 
@@ -1497,7 +1623,7 @@ def spam_filter(fp_in, user)
 	# /var/mail/#{user}פɵ롣
 	#
 	mailbox_file = AmConf['MailBoxFile'];
-	if is_in_white_list || spam_prob <= AmConf['SpamThreshold'].to_f
+	if result_mode == :is_white || result_mode == :is_clean
 		if mailbox_file != nil && ! mailbox_file.empty?
 			mfp = open(mailbox_file, "a");
 			exd.store_fp(mfp);
@@ -1529,6 +1655,8 @@ def spam_filter(fp_in, user)
 	rescue
 		# 㳰ȯƤ̵뤷˿ʤࡣ
 	end
+
+	return result;
 end
 
 # ======================================================================
@@ -1555,6 +1683,8 @@ class Mbox
 	# Żҥ᡼ΥƥȤե뤫ɤ߹ߡԥɤɸಽ롣
 	#
 	def load_fp(fp)
+		fp.set_encoding('ASCII-8BIT');
+					# ʳǤϥ󥳡ǥ󥰤
 		@mbuf_array = fp.readlines();
 		split_by_ufrom();
 	end
@@ -1563,7 +1693,12 @@ class Mbox
 	# Żҥ᡼ΥƥȤʸȤɤ߹ࡣ
 	#
 	def load_array(buf_array)
-		@mbuf_array = buf_array;
+		@mbuf_array = Array.new();
+		buf_array.each do |buf|
+			buf.force_encoding('ASCII-8BIT');
+					# ʳǤϥ󥳡ǥ󥰤
+			@mbuf_array << buf;
+		end
 		split_by_ufrom();
 	end
 
@@ -1580,7 +1715,7 @@ class Mbox
 		@unix_from_indexes = [ 0 ];	# From ׹Ԥʤѿ
 		prev_is_empty_line = true;
 		@mbuf_array.each_index do |n|
-			buf = @mbuf_array[n].toeuc;
+			buf = @mbuf_array[n];
 			if prev_is_empty_line && /^From / =~ buf
 				@unix_from_indexes << n;
 			end
